#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Calculate Pivot Points based on a user defined Higher Time Frame
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculate Pivot Points based on a user defined Higher Time Frame")]
    public class SupportAndResistanceMTF : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int hTFInterval = 1; // Default setting for HTFInterval
		private PeriodType hTFType = PeriodType.Day;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "S1"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "R1"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "S2"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "R2"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "S3"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "R3"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "PP"));
			
            Overlay				= true;
			
			if (BarsPeriod.Id!=hTFType || BarsPeriod.Value!=hTFInterval)
				Add(hTFType,hTFInterval);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			int index = BarsArray.Length-1;
			if (CurrentBars[index]<=0)
				return;
			
            double high = Highs[index][0];
			double low = Lows[index][0];
			double close = Closes[index][0];
			
			PP[0]				= (high + low + close) / 3;
			S1[0]				= 2 * PP[0] - high;
			R1[0]				= 2 * PP[0] - low;
			S2[0]				= PP[0] - (high - low);
			R2[0]				= PP[0] + (high - low);
			S3[0]				= PP[0] - 2 * (high - low);
			R3[0]				= PP[0] + 2 * (high - low);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries S1
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries R1
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries S2
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries R2
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries S3
        {
            get { return Values[4]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries R3
        {
            get { return Values[5]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries PP
        {
            get { return Values[6]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int HTFInterval
        {
            get { return hTFInterval; }
            set { hTFInterval = Math.Max(1, value); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public PeriodType HTFType
        {
            get { return hTFType; }
            set { hTFType = value; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SupportAndResistanceMTF[] cacheSupportAndResistanceMTF = null;

        private static SupportAndResistanceMTF checkSupportAndResistanceMTF = new SupportAndResistanceMTF();

        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        public SupportAndResistanceMTF SupportAndResistanceMTF(int hTFInterval, PeriodType hTFType)
        {
            return SupportAndResistanceMTF(Input, hTFInterval, hTFType);
        }

        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        public SupportAndResistanceMTF SupportAndResistanceMTF(Data.IDataSeries input, int hTFInterval, PeriodType hTFType)
        {
            if (cacheSupportAndResistanceMTF != null)
                for (int idx = 0; idx < cacheSupportAndResistanceMTF.Length; idx++)
                    if (cacheSupportAndResistanceMTF[idx].HTFInterval == hTFInterval && cacheSupportAndResistanceMTF[idx].HTFType == hTFType && cacheSupportAndResistanceMTF[idx].EqualsInput(input))
                        return cacheSupportAndResistanceMTF[idx];

            lock (checkSupportAndResistanceMTF)
            {
                checkSupportAndResistanceMTF.HTFInterval = hTFInterval;
                hTFInterval = checkSupportAndResistanceMTF.HTFInterval;
                checkSupportAndResistanceMTF.HTFType = hTFType;
                hTFType = checkSupportAndResistanceMTF.HTFType;

                if (cacheSupportAndResistanceMTF != null)
                    for (int idx = 0; idx < cacheSupportAndResistanceMTF.Length; idx++)
                        if (cacheSupportAndResistanceMTF[idx].HTFInterval == hTFInterval && cacheSupportAndResistanceMTF[idx].HTFType == hTFType && cacheSupportAndResistanceMTF[idx].EqualsInput(input))
                            return cacheSupportAndResistanceMTF[idx];

                SupportAndResistanceMTF indicator = new SupportAndResistanceMTF();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.HTFInterval = hTFInterval;
                indicator.HTFType = hTFType;
                Indicators.Add(indicator);
                indicator.SetUp();

                SupportAndResistanceMTF[] tmp = new SupportAndResistanceMTF[cacheSupportAndResistanceMTF == null ? 1 : cacheSupportAndResistanceMTF.Length + 1];
                if (cacheSupportAndResistanceMTF != null)
                    cacheSupportAndResistanceMTF.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSupportAndResistanceMTF = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SupportAndResistanceMTF SupportAndResistanceMTF(int hTFInterval, PeriodType hTFType)
        {
            return _indicator.SupportAndResistanceMTF(Input, hTFInterval, hTFType);
        }

        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        public Indicator.SupportAndResistanceMTF SupportAndResistanceMTF(Data.IDataSeries input, int hTFInterval, PeriodType hTFType)
        {
            return _indicator.SupportAndResistanceMTF(input, hTFInterval, hTFType);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SupportAndResistanceMTF SupportAndResistanceMTF(int hTFInterval, PeriodType hTFType)
        {
            return _indicator.SupportAndResistanceMTF(Input, hTFInterval, hTFType);
        }

        /// <summary>
        /// Calculate Pivot Points based on a user defined Higher Time Frame
        /// </summary>
        /// <returns></returns>
        public Indicator.SupportAndResistanceMTF SupportAndResistanceMTF(Data.IDataSeries input, int hTFInterval, PeriodType hTFType)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SupportAndResistanceMTF(input, hTFInterval, hTFType);
        }
    }
}
#endregion
